#!/usr/bin/env python
from copy import *
from inspect import *
from os.path import *
from sys import *
import sys

class TypeError(TypeError): pass
class Exception(Exception): pass

debug = False

def caller_modules():
    try:
        frames = getouterframes(currentframe())
        modules = []
        for frame,filename,line_number,function_name,lines,index in frames:
            module = getmodule(frame)
            if module and module not in modules:
                modules.append(module)
        return modules
    except IndexError:
        return []

def __all__debug(bool=True):
    global debug
    debug = bool

def append(module,name):
    global debug
    __all__ = module.__dict__.setdefault('__all__', [])
    if name not in __all__:  # Prevent duplicates if run from an IDE.
        __all__.append(name)
        if debug:
            print("%s.__all__ +%s" % (module.__name__,name))
        setattr(module,"__all__",sorted(__all__))

def isstring(object):
    try:
        int(object)
        return False
    except ValueError:
        return True
    except:
        return False

def publish2module(object,module,force=False):
    if module is None:
        err = "module is None"
        raise Exception(err)
    if not hasattr(module,"__file__"):
        # system module?
        err = "%s has no attribute __file__" % module
        raise Exception(err)
    file = module.__file__
    if isstring(object): # text
        name = object
        dir = dirname(file)
        if not hasattr(module,name) and not force:
            # not object name or not exists
            # object str value?
            for mname,member in getmembers(module):
                if id(member)==id(name):
                    return publish2module(mname,module)
        append(module,name)
        ismod = basename(file).find("__init__")!=0
        if ismod:
            # find parent package
            # iterate sys.modules
            for _,m in copy(modules).items():
                if not m or not hasattr(m,"__file__"):
                    continue
                ispkg = basename(m.__file__).find("__init__.py")==0
                if ispkg and dirname(m.__file__)==dir:
                    publish2module(name,m)
        return
    if isclass(object) or isroutine(object): 
        name = object.__name__
        return publish2module(name,module,force=True)
    # find object, compare by id
    for mname,member in getmembers(module):
        if id(member)==id(object):
            return publish2module(mname,module)
    # fix imp.load_module with custom name
    # search same module
    for k,m in modules.iteritems():
        if m and hasattr(m,"__file__") and m.__file__==file:
            module = m
            for mname,member in getmembers(module):
                if id(member)==id(object):
                    return publish2module(mname,module)
            for mname,member in getmembers(module):
                if member==object:
                    return publish2module(mname,module)
    # find object last try
    for mname,member in getmembers(module):
        if member==object:
            return publish2module(mname,module)
    err = "%s not exists in %s" % (object,module)
    raise Exception(err)

def public(*objects):
    """public decorator for __all__
    """
    modules = caller_modules()
    if len(modules)==1 or (objects and objects[0]==public):
        module = modules[0]
    else:
        modules = modules[1:] # exclude public
        module = modules[0]
    if not module: # error?
        if len(objects)==1:
            return objects[0]
        else:
            return objects
    for object in objects:
        if not hasattr(object,"__name__"):
            for k,v in module.__dict__.items():
                if v==object:
                    object=k
        if not (isclass(object) or isfunction(object) or isstring(object)):
            err = "@public expected class, function or str object name"
            raise TypeError(err)
        modname = module.__name__
        modnames = modname.split(".")
        for i,modname in enumerate(modnames):
            fullname = ".".join(modnames[0:i+1])
            if fullname in sys.modules:
                module = sys.modules[fullname]
                publish2module(object,module)
    if len(objects)==1:
        return objects[0]
    else:
        return objects

public(public)

if __name__=="__main__":
    print(__all__)
    
    # @public
    @public
    def func(): pass
    print(__all__)

    # public(*objects)
    def func2(): pass
    public(func2)
    print(__all__)
    public("func3")
    print(__all__)

    kwargs=dict(k="v")
    public(kwargs)
    print(__all__)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
__all__=["data_files"]
import imp
from os.path import *

repo = dirname(dirname(__file__))
if not repo: repo="."

def load_module(path):
    with open(path,'rb') as fp:
        # .hidden.py invisible for mdfind
        module = imp.load_module(path,fp,path,('.py', 'rb', imp.PY_SOURCE))  
        # __all__ required
        if not hasattr(module,'__all__'):
            raise ValueError("ERROR: %s __all__ required" % path)
        return module

data_files = [] 
path=join(repo,"data_files.py")
if exists(path):
	module = load_module(path)
	if hasattr(module,"data_files"):
		data_files = module.data_files
else:
    if __name__=="__main__":
        print("SKIP: %s NOT EXISTS" % path)

if __name__=="__main__":
	for k in __all__:
		if k in globals():
			print("%s: %s" % (k,globals()[k]))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
__all__=["description"]
from os.path import *

repo = dirname(dirname(__file__))
if not repo: repo="."

file = join(repo,"description")
if exists(file):
    description = open(file).read().lstrip().rstrip()
else:
    if __name__=="__main__":
        print("SKIP: description NOT EXISTS")

if __name__=="__main__":
	for k in __all__:
		if k in globals():
			print("%s: %s" % (k,globals()[k]))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
__all__=["py_modules","package_dir"]
import os
from os.path import *

repo = dirname(dirname(__file__))
if not repo: repo="."

path  = join(repo,"py_modules")
if exists(path) and isdir(path): # ./py_modules/
    listdir  = os.listdir(path)
    pyfiles = list(filter(lambda l:splitext(l)[1]==".py",listdir))
    pyfiles  = list(filter(lambda f:isfile(join(path,f)),pyfiles))
    py_modules  = list(map(lambda f:f.replace(".py",""),pyfiles))
    if py_modules:
        package_dir={'':"py_modules"}
else:
    if __name__=="__main__":
        print("SKIP: %s/ NOT EXISTS" % path)

if __name__=="__main__":
	for k in __all__:
		if k in globals():
			print("%s: %s" % (k,globals()[k]))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import imp
import os
from os.path import *
import sys
import warnings

# 1) distutils (Python Standart Library)
#   docs.python.org/2/distutils/setupscript.html
#   docs.python.org/3/distutils/setupscript.html
# python setup.py --manifest-only
# 2) setuptools (+easy_install tool)
#   pypi.python.org/pypi/setuptools
#   pythonhosted.org/setuptools/setuptools.html

# setuptools additional setup(name,**kwargs) keywords
setuptools_kwargs=[
    "include_package_data", # True/False
    "exclude_package_data", # True/False
    "package_data", # True/False
    "zip_safe", # True/False
    "install_requires", # [...]
    "entry_points", # [...]
    "extras_require", # [...]
    "setup_requires", # [...]
    "dependency_links", # [...]
    "namespace_packages", # [...]
    "test_suite", # ''
    "tests_require", # ''
    "test_loader" # ''
]
# setuptools `python setup.py` Extra commands (python setup.py --help):
setuptools_args=[
  "saveopts",          # save supplied options to setup.cfg or other config file
  "testr",             # Run unit tests using testr
  "develop",           # install package in 'development mode'
  "upload_docs",       # Upload documentation to PyPI
  "test",              # run unit tests after in-place build
  "setopt",            # set an option in setup.cfg or another config file
  "nosetests",         # Run unit tests using nosetests
  "install_egg_info",  # Install an .egg-info directory for the package
  "rotate",            # delete older distributions, keeping N newest files
  "bdist_mpkg",        # create a Mac OS X mpkg distribution for Installer.app
  "egg_info",          # create a distribution's .egg-info directory
  "py2app",            # create a Mac OS X application or plugin from Python scripts
  "alias",             # define a shortcut to invoke one or more commands
  "easy_install",      # Find/get/install Python packages
  "bdist_egg"          # create an "egg" distribution
]

def pyfiles(dir):
    list = os.listdir(dir)
    list = filter(lambda l:splitext(l)[1]==".py" and l.find("__")<0,list)
    return list

def module_kwargs(module):
    kwargs=dict()
    for k in getattr(module,"__all__"):
        if getattr(module,k):
            kwargs[k] = getattr(module,k)
    return kwargs

def load_module(path):
    with open(path,'rb') as fp:
        # .hidden.py invisible for mdfind
        module = imp.load_module(path,fp,path,('.py', 'rb', imp.PY_SOURCE))  
        # __all__ required
        if not hasattr(module,'__all__'):
            raise ValueError("ERROR: %s __all__ required" % path)
        return module

def update(**kwargs):
    for k,v in kwargs.items():
        if k not in sys.modules["__main__"].__all__:
            sys.modules["__main__"].__all__.append(k)
        setattr(sys.modules["__main__"],k,v)

def main():
    sys.modules["__main__"].__all__ = []
    dir = dirname(dirname(__file__))
    if not dir or dir==".": dir=os.getcwd()
    os.chdir(dir)
    sys.path+=[dir,dirname(__file__)]

    files = pyfiles(dirname(__file__))
    # RuntimeWarning: Parent module 'modname' not found while handling absolute import
    warnings.simplefilter("ignore", RuntimeWarning)

    for file in files:
        try:
            fullpath=join(dirname(__file__),file)
            module = load_module(fullpath)
            kwargs=module_kwargs(module)
            update(**kwargs)
            if len(sys.argv)==1 and len(kwargs)>0:
                print("%s: %s" % (file[1:],kwargs))
        except AttributeError: # variable from __all__ not initialized
            continue
    # ~/.setup_kwargs.py
    fullpath=join(os.environ["HOME"],".setup_kwargs.py")
    if exists(fullpath):
        module = load_module(fullpath)
        setup_kwargs = module_kwargs(module)

        update(**setup_kwargs)
        if len(sys.argv)==1 and len(setup_kwargs)>0: # debug
            print("%s: %s" % ("~/.setup_kwargs.py",setup_kwargs))

    def isstring(object):
        try:
            int(object)
            return False
        except ValueError:
            return True
        except:
            return False

    kwargs = module_kwargs(sys.modules["__main__"])
    if "name" in kwargs:
        name = kwargs["name"]
        del kwargs["name"]

    if len(sys.argv)==1 and kwargs: # debug
        print('\nsetup(name="%s",' % name)
        # for i,(k,v) in enumerate(sorted(kwargs.iteritems(),key=lambda (k,v):k),1): # python2
        for i,k in enumerate(sorted(list(kwargs.keys())),1): # python3
            v=kwargs[k]
            print("    %s = %s%s" % (k,'"%s"' % v if isstring(v) else v,"," if i!=len(kwargs) else ""))
        print(')')  

    setuptools=False # check for setuptools
    # `python setup.py` args
    for arg in setuptools_args:
        if arg in sys.argv:
            setuptools=True
    # setup(name,**kwargs) kwargs
    for arg in setuptools_kwargs:
        if arg in kwargs and kwargs[arg] is not None:
            v = kwargs[arg]
            if v!=[] and v!="" and v!={} and v!=False:
                setuptools=True

    if sys.argv[-1]=="--manifest-only": # distutils only
        setuptools=False

    from distutils.core import setup # default
    if setuptools:
        try:
            from setuptools import setup
        except ImportError:
            print("setuptools not installed") # use distutils
    if len(sys.argv)==1: return
    setup(name=name,**kwargs)

#if __name__=="__main__":
main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__all__ = []
import os, sys
from os.path import *

repo = dirname(__file__)
if not repo or repo==".": repo=os.getcwd()

# repo/setup.py 	- this file
# repo/.setup/*.py 	- python files imported by setup.py
setup = join(repo,".setup")
if exists(setup) and isdir(setup):
	sys.path.append(setup)
else:
	raise Exception("%s NOT EXISTS" % setup)

if __name__=="__main__":
	import __setup__



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__all__=["scripts"]
import os
from os.path import *

repo = dirname(dirname(__file__))
if not repo: repo="."

file = join(repo,"scripts.txt")
if exists(file) and isfile(file): # ./url.txt
    scripts = open(file).read().lstrip().rstrip().splitlines()
else:
    path  = join(repo,"scripts")
    if exists(path) and isdir(path):
        scripts = list(map(lambda name:join("scripts",name),
            list(filter(lambda f:isfile(join(path,f)) and f.find(" ")<0,
                os.listdir(path)
            ))
        ))
    else:
        if __name__=="__main__":
            print("SKIP: scripts NOT EXISTS")

if __name__=="__main__":
    for k in __all__:
        if k in globals():
            print("%s: %s" % (k,globals()[k]))
